package com.bocloud.mcs.aspect;

import com.megatron.framework.core.CurrentService;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.context.EnvironmentAware;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Component;

/**
 * 开关控制中心
 *
 * @author lym
 * @since 2022/07/18
 */
@Aspect
@Component
@RequiredArgsConstructor
@Slf4j
public class AccessAspect implements EnvironmentAware {

    private final CurrentService currentService;

    private Environment environment;

    @Override
    public void setEnvironment(Environment environment) {
        this.environment = environment;
    }

    @SneakyThrows
    @Around("@annotation(com.bocloud.mcs.aspect.EnableCollector)")
    public Object logAround(ProceedingJoinPoint joinPoint) {
        String methodName = joinPoint.getSignature().getName();
        EnableCollector annotation = joinPoint.getSignature().getDeclaringType().getMethod(methodName).getAnnotation(EnableCollector.class);
        String value = environment.resolvePlaceholders(annotation.value());
        if (Boolean.FALSE.equals(Boolean.valueOf(value))) {
            return false;
        }
        // 是否支持切片，如果不支持，则只允许主节点执行
        if (!annotation.section()) {
            if (!currentService.isLeader()) {
                log.debug("The current node is not the leader node");
                return false;
            }
        }
        return joinPoint.proceed();
    }
}
