package com.bocloud.mcs.collector;

import com.bocloud.mcs.service.internal.CmpCloudVendorService;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.moandjiezana.toml.Toml;
import io.prometheus.client.Collector;
import lombok.Getter;
import org.apache.commons.lang3.time.DateUtils;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.client.discovery.DiscoveryClient;

import java.util.*;

/**
 * 性能采集抽象类
 *
 * @author zhangyf
 */
public abstract class BocloudCollector extends Collector implements Collector.Describable, InitializingBean {

    protected static final String TIME = "time";
    protected static final String DATA = "data";

    @Autowired
    protected CmpCloudVendorService cmpCloudVendorService;
    @Autowired
    protected DiscoveryClient discoveryClient;
    @Autowired
    protected CurrentService currentService;
    @Autowired
    protected Service service;
    @Autowired
    @SuppressWarnings("SpringJavaInjectionPointsAutowiringInspection")
    protected Toml collectorConfig;

    /**
     * 性能指标缓存
     */
    @Getter
    private final Map<String, Object> cache = new HashMap<>();

    /**
     * 默认数据过期时间，10分钟（单位：秒）
     */
    protected static final int DEFAULT_EXPIRE = 600;

    /**
     * 数据有效时长，单位秒
     */
    @Getter
    private int expire = DEFAULT_EXPIRE;

    @SuppressWarnings("unchecked")
    @Override
    public List<MetricFamilySamples> collect() {
        if (getCache() != null && getCache().size() > 0) {
            Date now = new Date();
            Date time = (Date) getCache().get(TIME);
            // 数据过期
            if (now.getTime() > DateUtils.addSeconds(time, this.getExpire()).getTime()) {
                return new ArrayList<>();
            } else {
                return (List<MetricFamilySamples>) getCache().get(DATA);
            }
        } else {
            return new ArrayList<>();
        }
    }

    /**
     * 监控指标采集 定时任务
     */
    public abstract void scheduledCollect();

    @Override
    public List<MetricFamilySamples> describe() {
        return new ArrayList<>();
    }

    @Override
    public void afterPropertiesSet() {
        register();
    }

}
