package com.bocloud.mcs.collector.kvm;

import lombok.extern.slf4j.Slf4j;
import org.libvirt.Connect;
import org.libvirt.LibvirtException;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

@Slf4j
public class ConnectHolder {

    private static final Map<String, Connect> map = new HashMap<>();
    private static Lock lock = new ReentrantLock();

    private static boolean validate(String url, Connect connect) {
        if (connect == null) return false;
        try {
            if (!connect.isConnected()) return false;
            if (!connect.isAlive()) return false;
            connect.getHostName();
            return true;
        } catch (LibvirtException e) {
            log.error("connect {} is not avaliable, message: {}", url, e.getMessage(), e);
            return false;
        }
    }

    public static Connect getConnect(String url) throws LibvirtException {
        lock.lock();
        try {
            if (!validate(url, map.get(url))) {
                log.info("connect {} is not avaliable, recreate", url);
                map.put(url, new Connect(url));
            }
            log.debug("connect urls: {}", String.join(",", map.keySet()));
            return map.get(url);
        } finally {
            lock.unlock();
        }
    }
}
