package com.bocloud.mcs.collector.kvm;

import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.entity.KvmHost;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.mcs.aspect.EnableCollector;
import com.bocloud.mcs.collector.BocloudCollector;
import com.bocloud.mcs.collector.kvm.vm.KvmVmAdapter;
import io.prometheus.client.GaugeMetricFamily;
import lombok.extern.slf4j.Slf4j;
import org.libvirt.Connect;
import org.libvirt.Domain;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

/**
 * @author wpj
 * @since 2022/6/9
 */
@Component
@Slf4j
public class KvmVmCollector extends BocloudCollector {


    @Autowired
    private ApplicationContext applicationContext;

    @Value("${collector.kvm.thread.number:20}")
    private Integer threadNumber;

    @Override
    public List<MetricFamilySamples> collect() {
        return super.collect();
    }

    @Async
    @Override
    @EnableCollector(value = "${collector.kvm.vm.enable:true}")
    @Scheduled(initialDelay = 1000L, fixedRate = 60000)
    public void scheduledCollect() {
        log.info("collect kvm  metrics start...");
        GaugeMetricFamily collectTimeUse = new GaugeMetricFamily("kvm_" + "collect_time_use", "help",
                Arrays.asList("type"));
        ExecutorService executors = Executors.newFixedThreadPool(threadNumber);
        Date now = new Date();
        List<MetricFamilySamples> mfs = new ArrayList<>();
        List<MetricFamilySamples> list = new ArrayList<>();
        List<CloudVendor> vendors = cmpCloudVendorService.list(CloudProvider.KVM);
        List<KvmHost> hosts = new ArrayList<>();
        for (CloudVendor vendor : vendors) {
            List<KvmHost> kvmHosts = cmpCloudVendorService.listKvmHost(vendor.getId());
            hosts.addAll(kvmHosts);
        }

        for (KvmHost host : hosts) {
            executors.submit(() -> {
                Connect connect;
                try {
//                    connect = new Connect(host.getUri(), true);
                    connect = ConnectHolder.getConnect(host.getUri());
                    int[] idsOfDomain = connect.listDomains();
                    for (int id : idsOfDomain) {
                        Domain domain = connect.domainLookupByID(id);
                        if (domain.isActive() != 1) {
                            log.warn("domain 不处于激活状态 continue");
                            continue;
                        }
                        Map<String, KvmVmAdapter> adapters = applicationContext.getBeansOfType(KvmVmAdapter.class);
                        for (Map.Entry<String, KvmVmAdapter> kvmVmAdapterEntry : adapters.entrySet()) {
                            kvmVmAdapterEntry.getValue().getGaugeMetricFamilyData(domain, host.getVendorId(), list);
                        }
                    }
                } catch (Exception e) {
                    log.error("collect node metrics error!" + e.getMessage(), e);
                }
//                finally {
//                    if (connect != null) {
//                        try {
//                            connect.close();
//                        } catch (LibvirtException e) {
//                            log.error("连接关闭异常 {}", e);
//                        }
//                    }
//                }
            });
        }
        executors.shutdown();
        try {
            executors.awaitTermination(5, TimeUnit.MINUTES);
        } catch (InterruptedException e) {
            log.error("kvm not complete within its interval", e);
            Thread.currentThread().interrupt();
        }
        mfs.addAll(list);
        collectTimeUse.addMetric(Arrays.asList("instance"), (System.currentTimeMillis() - now.getTime()));
        mfs.add(collectTimeUse);
        super.getCache().put(TIME, new Date());
        super.getCache().put(DATA, mfs);
        log.info("collect kvm vm metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
    }

}

