package com.bocloud.mcs.collector.kvm.vm;

import io.prometheus.client.Collector;
import io.prometheus.client.GaugeMetricFamily;
import lombok.extern.slf4j.Slf4j;
import org.libvirt.Domain;
import org.libvirt.LibvirtException;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Slf4j
@Component
public class KvmVmCpuMetricData implements KvmVmAdapter {

    @Override
    public List<GaugeMetricFamily> getGaugeMetricFamilyData(Domain domain, Long vendorId, List<Collector.MetricFamilySamples> result) throws LibvirtException, InterruptedException {
        List<GaugeMetricFamily> list = new ArrayList<>();
        List<String> labels = Arrays.asList("vmCloud", "vm_uuid", "vmname");
        Double time = domain.getInfo().cpuTime / Math.pow(10, 9);
        long c1 = time.longValue();
        int vCpus = domain.getInfo().nrVirtCpu;
        log.info("monitoring data vmName=>{}, vmTime=>{},timeLongValue=>{},cpus=>{},systemNow=>{}"
                , domain.getName(), time, c1, vCpus, System.currentTimeMillis());
        GaugeMetricFamily cpuTimeData = new GaugeMetricFamily("kvm_" + "domain_info_cpu_time_seconds_total", "help",
                labels);
        cpuTimeData.addMetric(Arrays.asList(vendorId + "", domain.getUUIDString(), domain.getName()), c1);
        list.add(cpuTimeData);
        GaugeMetricFamily cpuMaxData = new GaugeMetricFamily("kvm_" + "domain_info_virtual_cpus", "help",
                labels);
        cpuMaxData.addMetric(Arrays.asList(vendorId + "", domain.getUUIDString(), domain.getName()), vCpus);

        GaugeMetricFamily cpuSystem = new GaugeMetricFamily("kvm_" + "domain_info_cpu_system_time", "help",
                labels);
        cpuSystem.addMetric(Arrays.asList(vendorId + "", domain.getUUIDString(), domain.getName()), System.currentTimeMillis() / Math.pow(10, 3));
        list.add(cpuMaxData);
        list.add(cpuSystem);
        result.addAll(list);
        return list;
    }

}
