package com.bocloud.mcs.collector.kvm.vm;

import com.alibaba.fastjson.JSONObject;
import io.prometheus.client.Collector;
import io.prometheus.client.GaugeMetricFamily;
import lombok.extern.slf4j.Slf4j;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.libvirt.Domain;
import org.libvirt.DomainBlockStats;
import org.libvirt.LibvirtException;
import org.springframework.stereotype.Component;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

@Slf4j
@Component
public class KvmVmDiskMetricData implements KvmVmAdapter {

    private String type = "disk";
    private static HashMap<String, String> typeMap = new HashMap();

    static {
        typeMap.put("rd_req", "读取请求总数");
        typeMap.put("rd_bytes", "读取的数据大小");
        typeMap.put("wr_req", "写入请求总数");
        typeMap.put("wr_bytes", "写入的数据大小");
        typeMap.put("errs", "失败次数");
    }


    @Override
    public List<GaugeMetricFamily> getGaugeMetricFamilyData(Domain domain, Long vendorId, List<Collector.MetricFamilySamples> result) throws LibvirtException, NoSuchFieldException, IllegalAccessException {
        List<GaugeMetricFamily> list = new ArrayList<>();
        List<String> labels = Arrays.asList("vmCloud", "vm_uuid", "vmname");
        String xmlDesc = domain.getXMLDesc(0);
        Document document = Jsoup.parse(xmlDesc);
        // 磁盘
        Elements disks = document.getElementsByTag("devices").get(0).getElementsByTag("disk");
        for (Element disk : disks) {
            String dev = disk.getElementsByTag("target").get(0).attr("dev");
            DomainBlockStats domainBlockStats = domain.blockStats(dev);
            log.debug("kvm disk {} stats {}", dev, JSONObject.toJSONString(domainBlockStats));
            Class<? extends DomainBlockStats> aClass = domainBlockStats.getClass();
            for (String str : typeMap.keySet()) {
                Field rxBytes = aClass.getField(str);
                long val = (long) rxBytes.get(domainBlockStats);
                GaugeMetricFamily nodeData = new GaugeMetricFamily("kvm_" + type + "_" + str, "help",
                        labels);
                nodeData.addMetric(Arrays.asList(vendorId + "", domain.getUUIDString(), domain.getName()), val);
                list.add(nodeData);
            }
        }
        result.addAll(list);
        return list;
    }


}
