package com.bocloud.mcs.collector.kvm.vm;

import com.alibaba.fastjson.JSONObject;
import io.prometheus.client.Collector;
import io.prometheus.client.GaugeMetricFamily;
import lombok.extern.slf4j.Slf4j;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.libvirt.Domain;
import org.libvirt.DomainInterfaceStats;
import org.libvirt.LibvirtException;
import org.springframework.stereotype.Component;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

@Component
@Slf4j
public class KvmVmInterfacesMetricData implements KvmVmAdapter {

    private String type = "interfaces";
    private static HashMap<String, String> typeMap = new HashMap();

    static {
        typeMap.put("rx_bytes", "接收数据包大小");
        typeMap.put("rx_packets", "接收数据包数量");
        typeMap.put("rx_errs", "接收错误数据包数量");
        typeMap.put("rx_drop", "接收丢弃数据包数量");
        typeMap.put("tx_bytes", "发送数据包大小");
        typeMap.put("tx_packets", "发送数据包数量");
        typeMap.put("tx_errs", "发送错误数据包数量");
        typeMap.put("tx_drop", "发送丢弃数据包数量");
    }

    @Override
    public List<GaugeMetricFamily> getGaugeMetricFamilyData(Domain domain, Long vendorId, List<Collector.MetricFamilySamples> result) throws LibvirtException, NoSuchFieldException, IllegalAccessException {
        List<GaugeMetricFamily> list = new ArrayList<>();
        List<String> labels = Arrays.asList("vmCloud", "vm_uuid");
        String xmlDesc = domain.getXMLDesc(0);
        Document document = Jsoup.parse(xmlDesc);
        // 网卡
        Elements interfaces = document.getElementsByTag("devices").get(0).getElementsByTag("interface");
        for (Element inter : interfaces) {
            String interName = inter.getElementsByTag("target").get(0).attr("dev");
            DomainInterfaceStats domainInterfaceStats = domain.interfaceStats(interName);
            log.debug("kvm interfaces {} stats  {}", interName, JSONObject.toJSONString(domainInterfaceStats));
            Class<? extends DomainInterfaceStats> aClass = domainInterfaceStats.getClass();
            for (String str : typeMap.keySet()) {
                Field rxBytes = aClass.getField(str);
                long val = (long) rxBytes.get(domainInterfaceStats);
                GaugeMetricFamily nodeData = new GaugeMetricFamily("kvm_" + type + "_" + str, "help",
                        labels);
                nodeData.addMetric(Arrays.asList(vendorId + "", domain.getUUIDString()), val);
                list.add(nodeData);
            }


        }
        result.addAll(list);
        return list;
    }


}

