package com.bocloud.mcs.collector.kvm.vm;

import com.alibaba.fastjson.JSONObject;
import io.prometheus.client.Collector;
import io.prometheus.client.GaugeMetricFamily;
import lombok.extern.slf4j.Slf4j;
import org.libvirt.Domain;
import org.libvirt.LibvirtException;
import org.libvirt.MemoryStatistic;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;

@Component
@Slf4j
public class KvmVmMemoryMetricData implements KvmVmAdapter {

    private String type = "memory_usage";

    @Override
    public List<GaugeMetricFamily> getGaugeMetricFamilyData(Domain domain, Long vendorId, List<Collector.MetricFamilySamples> result) throws LibvirtException {
        List<GaugeMetricFamily> list = new ArrayList<>();
        List<String> labels = Arrays.asList("vmCloud", "vm_uuid", "vmname");
        MemoryStatistic[] memoryStatistics = domain.memoryStats(10);
        Optional<MemoryStatistic> first = Arrays.stream(memoryStatistics).filter(x -> x.getTag() == 4).findFirst();
        double memoryUsage = 0;
        try {
            MemoryStatistic memoryStatistic = first.get();
            long unusedMemory = memoryStatistic.getValue();
            long maxMemory = domain.getMaxMemory();
            memoryUsage = (maxMemory - unusedMemory) * 100.0 / maxMemory;
            log.debug("虚拟机[{}]内存使用率为: {} ", domain.getName(), memoryUsage);
        } catch (Exception e) {
            log.error("first {}", JSONObject.toJSONString(first));
            log.error("获取内存异常{}", e);
        }

        GaugeMetricFamily nodeData = new GaugeMetricFamily("kvm_" + type, "help",
                labels);
        nodeData.addMetric(Arrays.asList(vendorId + "", domain.getUUIDString(), domain.getName()), (double) Math.round(memoryUsage * 100) / 100);
        list.add(nodeData);
        result.addAll(list);
        return list;
    }


}

