package com.bocloud.mcs.collector.manageone.bms;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.MonitorAuth;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.mcs.aspect.EnableCollector;
import com.bocloud.mcs.collector.BocloudCollector;
import com.bocloud.mcs.collector.manageone.bms.common.ManageOneBmsClient;
import com.bocloud.mcs.service.internal.CmsCloudVendorService;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.model.GeneralResult;
import io.prometheus.client.GaugeMetricFamily;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.util.*;
import java.util.stream.Collectors;

@Component
@Slf4j
public class BmsCollector extends BocloudCollector implements InitializingBean {

    private static final String PREFIX = "managerOne_bms_";
    @Autowired
    private CmsCloudVendorService cmsCloudVendorService;

    private Integer limit = 5;

    private static final Map<String,String> METRIC_MAP = new HashMap<>();

    static {
        METRIC_MAP.put("562992903094273","cpu_usage_user");   // 用户空间CPU使用率

        METRIC_MAP.put("562992903094285","mem_usedPercent");   // 内存使用率

        METRIC_MAP.put("562992903094292","disk_usedPercent");   // 磁盘使用率
        METRIC_MAP.put("562992903094296","disk_agt_read_bytes_rate");   // 磁盘读速率
        METRIC_MAP.put("562992903094297","disk_agt_read_requests_rate");   // 磁盘读操作速率
        METRIC_MAP.put("562992903094298","disk_agt_write_bytes_rate");   // 磁盘写速率
        METRIC_MAP.put("562992903094299","disk_agt_write_requests_rate");   // 磁盘写操作速率
        METRIC_MAP.put("562992903094301","disk_writeTime");   // 写操作平均耗时
        METRIC_MAP.put("562992903094302","disk_readTime");   // 读操作平均耗时

        METRIC_MAP.put("562992903094303","net_bitSent");   // 出网带宽
        METRIC_MAP.put("562992903094304","net_bitRecv");   // 入网带宽
        METRIC_MAP.put("562992903094305","net_packetSent");   // 网卡包发送速率
        METRIC_MAP.put("562992903094306","net_packetRecv");   // 网卡包接收速率
        METRIC_MAP.put("562992903094309","net_dropin");   // 接收丢包率
        METRIC_MAP.put("562992903094310","net_dropout");   // 发送丢包率
    }

    /**
     * 缓存过期时间，30分钟
     *
     * @return
     */
    @Override
    public int getExpire() {
        return 1800000;
    }

    @Async
    @Override
    @EnableCollector(value = "${collector.managerOne.bms.enable:true}")
    @Scheduled(initialDelay = 1000L, fixedRate = 120000L)
    public void scheduledCollect() {
        log.info("BMS => collect managerOne metrics start...");
        Date now = new Date();

        List<MetricFamilySamples> mfs = new ArrayList<>();
        GaugeMetricFamily collectTimeUse = new GaugeMetricFamily(PREFIX + "collect_time_use", "help",
                Collections.singletonList("type"));

        List<CloudVendor> vendors = cmpCloudVendorService.list(CloudProvider.MANAGEONE);

        for (CloudVendor vendor : vendors) {
                JSONObject auth = JSONObject.parseObject(vendor.getAuthentication());
                String projectId = auth.getString("projectId");
                ManageOneBmsClient manageOneBmsClient = new ManageOneBmsClient();
                GeneralResult authResult = cmsCloudVendorService.detailVendorAuth(vendor.getId());
                Assert.isTrue(authResult.isSuccess(), "get vendor operations auth failed , error:" + authResult.getMessage());
                MonitorAuth monitorAuth = JSONObject.parseObject(JSONObject.toJSONString(authResult.getData()), MonitorAuth.class);

                Assert.notNull(monitorAuth, "vendor [" + vendor.getName() + "] not found auth config info");
                String password = new AESEncryptor().decrypt(monitorAuth.getPassword(), null);

                manageOneBmsClient.setAddr(monitorAuth.getAddress());
                manageOneBmsClient.setUserName(monitorAuth.getUsername());
                manageOneBmsClient.setPassword(password);
                manageOneBmsClient.setProjectId(projectId);
                Integer offset = 0;
                JSONArray jsonArray = manageOneBmsClient.getVms(limit, offset);
                while (null != jsonArray && jsonArray.size() != 0){
                    // 每五台机器循环获取一次监控数据
                    offset += limit;
                    List<JSONObject> vmJson = jsonArray.toJavaList(JSONObject.class);
                    List<String> vmResId = vmJson.stream().map(jsonObject -> jsonObject.getString("resId")).collect(Collectors.toList());
                    JSONObject monitor = manageOneBmsClient.getMonitor(vmResId, METRIC_MAP);
                    if (null != monitor){
                        JSONObject data = monitor.getJSONObject("data");
                        if (null != data){
                            for (String vmMonitorStr : data.keySet()) {
                                // 单个云主机监控数据，key为监控指标的key vmMonitorStr为objId
                                JSONObject vmMonitor = data.getJSONObject(vmMonitorStr);
                                for (String monitorKey : vmMonitor.keySet()) {
                                    if (METRIC_MAP.containsKey(monitorKey)){
                                        JSONObject monitorJSONObject = vmMonitor.getJSONObject("monitorKey");
                                        GaugeMetricFamily cpuMaxData = new GaugeMetricFamily(PREFIX + METRIC_MAP.get(monitorKey), "help",
                                                Arrays.asList("vendorId", "instanceId"));
                                        cpuMaxData.addMetric(Arrays.asList(vendor.getId() + "", vmMonitorStr), monitorJSONObject.getFloatValue("last_collect_data"));
                                        mfs.add(cpuMaxData);
                                    }
                                }
                            }
                        }else {
                            log.info("本次获取到监控数据为null");
                        }
                    }
                    jsonArray = manageOneBmsClient.getVms(limit, offset);
                }

        }


        collectTimeUse.addMetric(Collections.singletonList("manager_bms"), (System.currentTimeMillis() - now.getTime()));
        mfs.add(collectTimeUse);
        super.getCache().put(TIME, new Date());
        super.getCache().put(DATA, mfs);
        log.info("collect  manager bms metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
    }
}
