package com.bocloud.mcs.collector.manageone.bms.common;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.http.HttpClient;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Data
@AllArgsConstructor
@NoArgsConstructor
@Slf4j
public class ManageOneBmsClient {
    private static final String BATCH_GET_METRIC_DATA_URL = "/rest/performance/v1/data-svc/latest-data/action/query";
    private static final String BMS_OBJ_TYPE_ID = "562992903094272";
    private static final String VM_LIST_URL = "/rest/tenant-resource/v1/tenant/resources/CLOUD_BMS";
    private static final String TOKEN_URL = "/rest/plat/smapp/v1/sessions";


    private HttpClient client = new HttpClient(PostDataFormat.RAW);

    private String userName;
    private String password;
    private String addr;
    private String projectId;



    public String getToken(){
        ManageCommon manageCommon = new ManageCommon();
        Map<String, Object> params = new HashMap<>();
        params.put("grantType", "password");
        params.put("userName", this.userName);
        params.put("value" ,this.password);
        log.info("BMS => 获取token接口参数为{}", JSONObject.toJSONString(params));
        JSONObject jsonObject = manageCommon.put(this.buildUrl(TOKEN_URL), buildHeader(null), params);
        log.info("BMS => 获取token接口返回: {}", jsonObject.toJSONString());
        log.info("BMS => token为: {}", jsonObject.getString("accessSession"));
        return jsonObject.getString("accessSession");
    }

    private String buildUrl(String tokenUrl) {
        return this.addr + tokenUrl;
    }

    private Map<String, Object> buildHeader(String token) {
        HashMap<String, Object> header = new HashMap<>();
        header.put("Content-Type", "application/json;charset=UTF-8");
        header.put("Accept", "application/json");
        if (null != token){
            header.put("x-auth-token", token);
        }
        return header;
    }


    public JSONArray getVms(Integer limit, Integer offset) {
        Map<String, Object> header = this.buildHeader(this.getToken());
        ManageCommon manageCommon = new ManageCommon();
        HashMap<String, Object> params = new HashMap<>();
        params.put("limit", limit);
        params.put("offset", offset);
        log.info("BMS => 获取bms列表参数为:{}, url为:{}, 请求头为{}", JSONObject.toJSONString(params), buildUrl(VM_LIST_URL),
                JSONObject.toJSONString(header));
        try {
            JSONObject jsonObject = manageCommon.get(buildUrl(VM_LIST_URL), header, params);
            log.info("BMS => 获取到bms列表为{}", jsonObject.toJSONString());
            return jsonObject.getJSONArray("objList");
        }catch (Exception e){
            log.error("获取bms 云主机列表异常", e);
            return null;
        }
    }

    public JSONObject getMonitor(List<String> vmIds, Map<String,String> METRIC_MAP) {
        Map<String, Object> header = this.buildHeader(this.getToken());
        ManageCommon manageCommon = new ManageCommon();
        HashMap<String, Object> params = new HashMap<>();
        params.put("obj_type_id", BMS_OBJ_TYPE_ID);
        params.put("indicator_ids", JSONObject.parseArray(JSONObject.toJSONString(new ArrayList<>(METRIC_MAP.keySet()))));
        params.put("obj_ids", JSONObject.parseArray(JSONObject.toJSONString(vmIds)));
        log.info("BMS => 获取bms监控参数为{}", JSONObject.toJSONString(params));
        try {
            JSONObject jsonObject = manageCommon.post(buildUrl(BATCH_GET_METRIC_DATA_URL), header, params);
            log.info("BMS => 获取到bms监控数据为 {}", jsonObject.toJSONString());
            return jsonObject;
        }catch (Exception e){
            log.error("BMS => 获取bms 云主机列表异常", e);
            return null;
        }
    }




}
