package com.bocloud.mcs.collector.manageone.cce;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.MonitorAuth;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.mcs.aspect.EnableCollector;
import com.bocloud.mcs.collector.BocloudCollector;
import com.bocloud.mcs.collector.manageone.cce.common.ContainerCluster;
import com.bocloud.mcs.collector.manageone.cce.common.ManageOneCceClient;
import com.bocloud.mcs.collector.manageone.cce.common.ManagerMonitorEntity;
import com.bocloud.mcs.common.Authentication;
import com.bocloud.mcs.service.internal.CmsCloudVendorService;
import com.bocloud.mcs.service.internal.ImsCloudServerService;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.model.GeneralResult;
import io.prometheus.client.GaugeMetricFamily;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.util.*;
import java.util.stream.Collectors;

@Component
@Slf4j
public class CceCollector extends BocloudCollector implements InitializingBean {

    private static final String PREFIX = "managerOne_cce_";
    @Autowired
    private CmsCloudVendorService cmsCloudVendorService;
    @Autowired
    private ImsCloudServerService imsCloudServerService;

    private Integer limit = 5;

    private static final Map<String,String> METRIC_MAP = new HashMap<>();

    static {
//        METRIC_MAP.put("磁盘写入速率","diskWriteRate");   // 用户空间CPU使用率
//        METRIC_MAP.put("磁盘读取速率","diskReadRate");   // 内存使用率
//        METRIC_MAP.put("CPU使用率","cpuUsage");   // 磁盘使用率
//        METRIC_MAP.put("物理内存使用率","memUsage");   // 磁盘读速率
//        METRIC_MAP.put("接收Bps","recvBytesRate");   // 磁盘读操作速率
//        METRIC_MAP.put("接收Pps","recvPackRate");   // 磁盘写速率
//        METRIC_MAP.put("显存使用率","gpuMemUsage");   // 磁盘写操作速率
//        METRIC_MAP.put("GPU使用率","gpuUtil");   // 写操作平均耗时
//        METRIC_MAP.put("发送Bps","sendBytesRate");   // 写操作平均耗时
//        METRIC_MAP.put("发送Pps","sendPackRate");   // 写操作平均耗时



        METRIC_MAP.put("CPU使用率","cpuUsage");
        METRIC_MAP.put("内存使用率","memUsedRate");

        METRIC_MAP.put("磁盘使用率","diskUsedRate");
        METRIC_MAP.put("网卡接收速率","recvBytesRate");   // Bytes/Second
        METRIC_MAP.put("网卡发送速率","sendBytesRate");   // Bytes/Second
    }

    /**
     * 缓存过期时间，30分钟
     *
     * @return
     */
    @Override
    public int getExpire() {
        return 1800000;
    }

    @Async
    @Override
    @EnableCollector(value = "${collector.managerOne.cce.enable:true}")
    @Scheduled(initialDelay = 1000L, fixedRate = 120000L)
    public void scheduledCollect() {
        log.info("CCE => collect managerOne metrics start...");
        Date now = new Date();

        List<MetricFamilySamples> mfs = new ArrayList<>();
        GaugeMetricFamily collectTimeUse = new GaugeMetricFamily(PREFIX + "collect_time_use", "help",
                Collections.singletonList("type"));

        List<CloudVendor> vendors = cmpCloudVendorService.list(CloudProvider.MANAGEONE);

        for (CloudVendor vendor : vendors) {
            try {
                Authentication authentication = JSON.parseObject(vendor.getAuthentication(), Authentication.class);
                GeneralResult authResult = cmsCloudVendorService.detailVendorAuth(vendor.getId());
                Assert.isTrue(authResult.isSuccess(), "get vendor operations auth failed , error:" + authResult.getMessage());
                MonitorAuth monitorAuth = JSONObject.parseObject(JSONObject.toJSONString(authResult.getData()), MonitorAuth.class);
                Assert.notNull(monitorAuth, "vendor [" + vendor.getName() + "] not found auth config info");
                String addr = monitorAuth.getAddress().replace("oc", "aom");
                log.info("aom地址为: {}", addr);
                String password = new AESEncryptor().decrypt(authentication.getPassword(), "BocloudCMP");
                ManageOneCceClient manageOneCceClient = new ManageOneCceClient(authentication.getAddress(), addr,authentication.getAccount(), password, authentication.getDomainName(), authentication.getProjectId());

                List<ContainerCluster> cceContainer = imsCloudServerService.getCceContainer(vendor.getId());
                List<String> clusterUuids = cceContainer.stream().map(ContainerCluster::getClusterUuid).collect(Collectors.toList());
                log.info("CCE => 获取到cce列表为{}", JSONObject.toJSONString(cceContainer));
                for (String value : METRIC_MAP.values()) {
                    for (String clusterUuid : clusterUuids) {
                        JSONObject monitorResource = manageOneCceClient.getMonitor(Arrays.asList(clusterUuid), value);
                        if (monitorResource!= null) {
                            ManagerMonitorEntity managerMonitorEntity = JSONObject.parseObject(monitorResource.toJSONString(), ManagerMonitorEntity.class);
                            for (ManagerMonitorEntity.Metrics metric : managerMonitorEntity.getMetrics()) {

                                List<ManagerMonitorEntity.Dimensions> dimensions = metric.getMetric().getDimensions();
                                List<ManagerMonitorEntity.DataPoints> dataPoints = metric.getDataPoints();

                                if (dimensions!= null && dimensions.size() > 0) {
                                    for (int i = 0; i < dimensions.size(); i++) {
                                        if (clusterUuid.contains(dimensions.get(i).getValue())){
                                            String cceUuid = dimensions.get(i).getValue();
                                            String val = dataPoints.get(i).getStatistics().get(0).getValue();
                                            GaugeMetricFamily cpuMaxData = new GaugeMetricFamily(PREFIX + metric.getMetric().getMetricName(), "help",
                                                    Arrays.asList("vendorId", "instanceId"));
                                            cpuMaxData.addMetric(Arrays.asList(vendor.getId() + "", cceUuid), Double.parseDouble(val));
                                            mfs.add(cpuMaxData);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }catch (Exception e) {
                log.error("collect managerOne metrics error", e);
            }
        }

        collectTimeUse.addMetric(Collections.singletonList("manager_cce"), (System.currentTimeMillis() - now.getTime()));
        mfs.add(collectTimeUse);
        super.getCache().put(TIME, new Date());
        super.getCache().put(DATA, mfs);
        log.info("collect  manager bms metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
    }
}
