package com.bocloud.mcs.collector.manageone.cce.common;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.encrypt.Encryptor;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.DateDeserializer;
import com.megatron.common.utils.DateSerializer;
import com.megatron.common.utils.DaySerializer;
import lombok.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * @ClassName: ContainerCluster
 * @Description: 容器Kubernetes 集群
 * @Author lxy
 * @Date 2022/7/14 17:54
 */
@Slf4j
@Data
@NoArgsConstructor
@AllArgsConstructor
public class ContainerCluster  {

    private Long id;// ID
    private String regionId;// 所属域

    private Long vendorId;// 所属云平台
    private String clusterUuid;// 集群ID
    private String clusterType;// 集群类型
    // 集群创建时间
    private Date created;
    // 集群版本
    private String version;
    // 集群Docker版本。
    private String dockerVersion;
    // 集群API Server访问地址，包含内网访问地址以及公网访问地址。
    private String masterUrl;
    // 集群安全组ID。
    private String securityGroupUuid;
    /**
     * 集群运行状态，取值： initial：集群创建中。 failed：集群创建失败。 running：集群运行中。 updating：集群升级中。 updating_failed：集群升级失败。 scaling：集群伸缩中。
     * stopped：集群已经停止运行。 deleting：集群删除中。 deleted：集群已经被删除。 delete_failed：集群删除失败。
     */
    // @Column("status")
    // private String status;

    /**
     * Pod网络地址段，必须是有效的私有网段，即以下网段及其子网：
     * <p>
     * 10.0.0.0/8 172.16-31.0.0/12-16 192.168.0.0/16 不能与VPC及VPC内已有Kubernetes集群使用的网段重复，创建成功后不能修改。
     */
    private String subnetCidr;
    // 集群专有网络ID，创建集群时必传。
    private String vpcUuid;
    private String vpcName;
    // 虚拟交换机ID，创建集群时可选择1~3个虚拟交换机。为保证集群高可用，建议选择不同可用区的虚拟交换机。
    private String vswitchUuid;
    // 集群所属可用区ID。
    private String zoneId;
    // 托管版集群类型，面向托管集群： ack.pro.small：专业托管集群。 ack.standard：标准托管集群。
    private String clusterSpec;
    // 集群删除保护
    private Boolean deletionProtection;
    private String eipUuid;
    private String loadBalancerUuid;
    private String resourceGroupUuid;// 资源组id
    // SSH登录密码。密码规则为8~30 个字符，且至少同时包含三项（大小写字母、数字和特殊符号），和key_pair二选一。
    public String loginPassword;
    // 密钥对名称，和login_password二选一。
    public String keyPair;

    // 网络插件
    private String capabilitiesNetwork;
    // kube-proxy代理模式
    private String capabilitiesProxyModel;

    // HSC START
    // 子网uuid
    private String subnetUuid;
    private String subnetName;
    // 容器网段
    private String containerCidr;
    // 服务网段
    private String serviceCidr;
    // HCS END

    // 自服务 start
    private Long tenantId;// 租户ID
    private String tenantName; // 租户名称

    private Long projectId;// 项目ID
    private String projectName; // 项目名称

    private String uuid;// uuid，对应子订单号

    @JsonSerialize(using = DaySerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date expiredTime;// 过期时间
    // 所属申请类别
    private RequestContext.Catalog catalog;
    // 所属业务id
    private Long businessId;
    // 所属业务名称
    private String businessName;

    private Boolean isRecycle = false;
    @JsonSerialize(using = DaySerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date recycleTime;// 回收时间
    // 自服务 end

    private String regionName;
    private String vpcSafetyType;//安全类型
    private String vpcFunctionType;
    private String applicationName; //应用系统
    private String resourceSetName; //资源集
    private String imageId;//自定义镜像


    @Getter(AccessLevel.NONE)
    @Setter(AccessLevel.NONE)
    private Encryptor encryptor = new AESEncryptor();


    @JsonSerialize(
            using = DateSerializer.class
    )
    @JsonDeserialize(
            using = DateDeserializer.class
    )
    @DateTimeFormat(
            pattern = "yyyy-MM-dd HH:mm:ss"
    )
    private Date gmtCreate;

    @JsonSerialize(
            using = DateSerializer.class
    )
    @JsonDeserialize(
            using = DateDeserializer.class
    )
    @DateTimeFormat(
            pattern = "yyyy-MM-dd HH:mm:ss"
    )
    private Date gmtModify;
    private Long creatorId;
    private Long ownerId;
    private Long menderId;
    private String name;
    private String status;
    private Boolean deleted;
    private String props;
    private String remark;

}
