package com.bocloud.mcs.collector.manageone.cce.common;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.http.CommonResultBuilder;
import com.megatron.common.http.HttpClient;
import com.megatron.common.model.Result;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.Header;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;

import java.util.*;

/**
 * @ Description： @ Author ：丁明威 @ Date ：Created in 17:07 2018/11/14 @ Modified
 * By：
 */
@Data
@Slf4j
@NoArgsConstructor
public class ManageOneCceClient {

    private static final String GET_METRIC_DATA_URL = "/v1/{project_id}/ams/metricdata";

    private String username;// 管理员账号
    private String password;// 管理员密码
    private String domain;// 所属域
    private String project;// 项目ID
    private boolean accessable;
    private ManageOneToken token;// token信息
    private String error; // 错误信息
    private HttpClient client = new HttpClient(60 * 1000, PostDataFormat.RAW);
    private String monitorUrl;
    private String monitorUserName;
    private String monitorPwd;
    private String monitorToken;
    private String monitorRoaRand;
    private String tokenUrl;


    public ManageOneCceClient(String tokenUrl,String monitorUrl, String userName, String pwd, String domainName, String projectId) {
        this.tokenUrl = tokenUrl;
        this.monitorUrl = monitorUrl;
        this.username = userName;
        this.password = pwd;
        this.domain = domainName;
        this.project = projectId;
        initToken();
    }

    private void initToken() {
        init();
    }


    /**
     * 初始化连接
     */
    private void init() {
        String url = this.tokenUrl + "/v3/auth/tokens";
        ManageOneDomain domain = new ManageOneDomain(this.domain);
        ManageOneUser user = new ManageOneUser(this.username, this.password, domain);
        ManageOnePassword password = new ManageOnePassword(user);
        List<String> methods = new ArrayList<String>();
        methods.add("password");
        ManageOneIdentity identity = new ManageOneIdentity(password, methods);

        ManageOneProject project = new ManageOneProject(this.project);
        ManageOneScope scope = new ManageOneScope(project);
        ManageOneAuth auth = new ManageOneAuth(identity, scope);
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("auth", auth);
        HttpPost httpPost = new HttpPost(url);
        httpPost.addHeader("accept", "application/json");
        httpPost.addHeader("content-type", "application/json");
        log.info("CCE => 参数{}", JSONObject.toJSONString(params));
        StringEntity httpEntity = new StringEntity(JSONObject.toJSON(params).toString(), "UTF-8");
        httpPost.setEntity(httpEntity);
        try {
            CloseableHttpResponse httpResponse = client.getHttpClient().execute(httpPost);
            log.info("CCE => 获取token地址{},参数{}, 返回{}", url, httpEntity, JSONObject.toJSONString(httpResponse));
            Result result = new CommonResultBuilder().build(httpResponse.getStatusLine().getStatusCode());
            if (result.isFailed()) {
                this.accessable = false;
                log.error("CCE => Driver init error : {}", result.getMessage());
                this.error = result.getMessage();
                return;
            }
            JSONObject content = JSONObject
                    .parseObject(IOUtils.toString(httpResponse.getEntity().getContent(), "utf-8"));
            this.token = JSONObject.parseObject(content.getString("token"), ManageOneToken.class);
            if (null == this.token) {
                this.accessable = false;
                log.error("CCE => Driver init error : {}", "Token is empty !");
                this.error = "Token is empty !";
                return;
            }
            String authToken = findToken(httpResponse);
            if (StringUtils.isEmpty(authToken)) {
                this.accessable = false;
                log.error("CCE => Driver init error : {}", "Token is empty !");
                this.error = "CCE => Token is empty !";
                return;
            }
            token.setToken(authToken);
            this.accessable = true;
            log.info("CCE => Driver init token : {}", JSON.toJSONString(token));
        } catch (Exception e) {
            log.error("CCE => Get error message：", e);
            this.accessable = false;
        }
    }

    /**
     * 从响应头部获取会话X-Subject-Token
     *
     * @param httpResponse
     * @return
     */
    private String findToken(CloseableHttpResponse httpResponse) {
        String token = "";
        Header[] headers = httpResponse.getHeaders("X-Subject-Token");
        if (null == headers || headers.length == 0) {
            return token;
        }
        for (Header header : headers) {
            if ("X-Subject-Token".equalsIgnoreCase(header.getName())) {
                token = header.getValue();
                break;
            }
        }
        return token;
    }




    private Map<String, Object> buildHeader() {
        Map<String, Object> header = new HashMap<String, Object>();
        header.put("accept", "application/json");
        header.put("content-type", "application/json");
        header.put("X-Auth-Token", this.token.getToken());
        return header;
    }

    public JSONObject getMonitor(List<String> instanceIds, String metricName) {
        ManageOneCommon manageOneCommon = new ManageOneCommon();
        String dimensionName = "clusterId";// 维度名称。长度最短为1，最大为32。
        String timerange = "-1.-1.5"; // 表示查询近5分钟数据
        long period = 60; //周期
        JSONObject metricObj = new JSONObject();
        metricObj.put("namespace", "PAAS.AGGR");
        metricObj.put("metricName", metricName);
        ArrayList<Object> dimensions = new ArrayList<>();
        instanceIds.forEach(instanceid -> {
            JSONObject dimension = new JSONObject();
            dimension.put("name", dimensionName);
            dimension.put("value", instanceid);
            dimensions.add(dimension);
        });
        metricObj.put("dimensions", dimensions);
        Map<String, Object> params = new HashMap<>();
        params.put("metrics", Arrays.asList(metricObj));
        params.put("period", period);
        params.put("timerange", timerange);
        params.put("statistics", Collections.singletonList("average"));
        String url = buildUrl(GET_METRIC_DATA_URL, this.project);
        log.info("CCE => 获取cce监控参数为{}, 路径为: {}, 参数为{}", JSONObject.toJSONString(params), url, JSONObject.toJSONString(params));
        Map<String, Object> header = this.buildHeader();
        JSONObject jsonObject = manageOneCommon.post(url, header, params);
        log.info("CCE => 获取到cce监控数据为 {}", jsonObject.toJSONString());
        return jsonObject;
    }

    private String buildUrl(String url, String projectId) {
        return (this.monitorUrl + url).replace("{project_id}", projectId);
    }


}