package com.bocloud.mcs.collector.tianyicloud.core;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.megatron.common.enums.HttpRequestMethod;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.http.CommonResultBuilder;
import com.megatron.common.http.HttpClient;
import com.megatron.common.http.ResultBuilder;
import com.megatron.common.model.Result;
import com.megatron.common.utils.ListTool;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.Assert;

import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Data
public class TianyiProvider {
    private String accessKey;
    private String secretKey;
    private String projectId;
    private String ctUserId;
    private String regionId;
    public static final Integer SUCCESS_STATUS_CODE = 800;
    public String PROTOCOL;
    public Integer PAGE_SIZE = 50;
    private final HttpClient client = new HttpClient(60 * 1000, PostDataFormat.RAW);

    protected ResultBuilder resultBuilder = new CommonResultBuilder();

    public TianyiProvider(String ak, String sk, String projectId, String reginId, String url) {
        this.accessKey = ak;
        this.secretKey = sk;
        this.ctUserId = projectId;
        this.regionId = reginId;
        this.PROTOCOL = url;
    }

    private Map<String, Object> buildHeader(HttpRequestMethod method, Map<String, Object> headers, Map<String, Object> params) {
        return TianyiHeaderUtils.createHeaders(method, headers, params, this.accessKey, this.secretKey, this.ctUserId);
    }

    public String buildUrl(String apiUrl) {
        return PROTOCOL + apiUrl;
    }

    public Result doGet(String url, Map<String, Object> headers) {
        HttpClient httpClient = new HttpClient(20 * 1000, PostDataFormat.RAW);
        httpClient.setDataFormat(null);
        return httpClient.get(buildHeader(HttpRequestMethod.GET, headers, null), null, buildUrl(url));
    }

    public Result doGet(String url, Map<String, Object> headers, Map<String, Object> params) {
        HttpClient httpClient = new HttpClient(60 * 1000, PostDataFormat.RAW);
        httpClient.setDataFormat(null);

        String uri = buildUrl(url);
        Map<String, Object> header = buildHeader(HttpRequestMethod.GET, headers, params);
        printCurl(HttpRequestMethod.GET, uri, header, params);
        return httpClient.get(header, params, uri);
    }

    public Result doPost(String url, Map<String, Object> params) {
        HttpClient httpClient = new HttpClient(60 * 1000, PostDataFormat.RAW);
        Map<String, Object> header = buildHeader(HttpRequestMethod.POST, null, params);
        printCurl(HttpRequestMethod.POST, buildUrl(url), header, params);
        return httpClient.post(header, params, buildUrl(url));
    }


    /**
     * 用于请求回调的check
     *
     * @param result 结果
     * @param title  接口名
     */
    public JSONObject checkResult(Result result, String title) {
        Assert.isTrue(result.isSuccess(), "调用" + title + "API失败" + result.getMessage());
        JSONObject object = JSONObject.parseObject(result.getMessage());
        Assert.notNull(object, "调用" + title + "API" + "返回的object为null");
        log.debug("API返回的数据:{}", object.toJSONString());
        Assert.isTrue(object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE), "调用-" + title + "-API-返回的statusCode为" + object.getInteger("statusCode") + "-message:" + object.getString("message") + "-description:" + object.getString("description"));
        JSONObject returnObj = object.getJSONObject("returnObj");
        log.debug("returnObj返回的数据:{}", returnObj == null ? null : returnObj.toJSONString());
        return returnObj;
    }

    public void printCurl(HttpRequestMethod method, String uri, Map<String, Object> header, Map<String, Object> params) {
        StringBuilder builder = new StringBuilder();
        builder.append("curl --location --request");
        if (Objects.requireNonNull(method) == HttpRequestMethod.GET) {
            builder.append(" GET ").append("'").append(uri);

            if (params == null || params.isEmpty()) {
                builder.append("'");
            } else {
                List<String> keyList = new ArrayList<>(params.keySet());
                keyList = keyList.stream().sorted(Comparator.comparing(String::toString)).toList();
                // aa=1&bb=2
                StringBuilder queryBuffer = new StringBuilder();
                for (String key : keyList) {
                    if (!queryBuffer.isEmpty()) {
                        queryBuffer.append('&');
                    }
                    Object value = params.get(key);
                    if (value == null) {
                        value = "";
                    }
                    String encodedValue = URLEncoder.encode(String.valueOf(value), StandardCharsets.UTF_8);
                    queryBuffer.append(key).append('=').append(encodedValue);
                }
                String query = queryBuffer.toString();
                builder.append("?").append(query).append("'");
            }
        } else {
            builder.append(" POST ").append("'").append(uri);
            if (params == null || params.isEmpty()) {
                builder.append("'");
            } else {
                builder.append("'");
                builder.append(" --data-raw ").append("'{");
                List<String> keyList = new ArrayList<>(params.keySet());
                StringBuilder queryBuffer = new StringBuilder();
                for (String key : keyList) {
                    if (!queryBuffer.isEmpty()) {
                        queryBuffer.append(',');
                    }
                    Object value = params.get(key);
                    if (value == null) {
                        value = "";
                    }
                    String encodedValue = String.valueOf(value);
                    queryBuffer.append("\"").append(key).append("\"").append(':').append("\"").append(encodedValue).append("\"");
                }
                builder.append(queryBuffer);
                builder.append("}'");
            }
        }

        for (Map.Entry<String, Object> entry : header.entrySet()) {
            builder.append(" --header ").append("'").append(entry.getKey()).append(":").append(entry.getValue()).append("'");
        }
        uri = builder.toString();
        log.debug("uri:{}", uri);
    }


    public List<JSONObject> getMetric(List<String> instanceList) {
        String url = "/v4.1/monitor/query-vm-latestmetricdata";
        JSONObject body = new JSONObject();
        body.put("regionID", this.regionId);
        body.put("deviceUUIDList", instanceList);

        HttpClient httpClient = new HttpClient(60 * 1000, PostDataFormat.RAW);
        String uri = buildUrl(url);
        Map<String, Object> header = buildHeader(HttpRequestMethod.POST, null, body);
        printCurl(HttpRequestMethod.POST, uri, header, body);
        Result result = httpClient.post(header, body, uri);
        if (JSONObject.parseObject(result.getMessage()).getInteger("statusCode").equals(800)) {
            return JSONObject.parseArray(JSONObject.parseObject(result.getMessage()).getJSONObject("returnObj").getJSONArray("result").toJSONString(), JSONObject.class);
        } else {
            throw new RuntimeException("获取监控数据失败，接口返回 " + JSONObject.toJSONString(result));
        }
    }


    public List<String> instanceList() {
        int pageNo = 1;
        try {
            String apiUrl = "/v4/ecs/instance-list";

            JSONObject body = new JSONObject();
            body.put("regionID", this.getRegionId());
            body.put("pageNo", pageNo);
            body.put("pageSize", this.PAGE_SIZE);
            HttpClient httpClient = new HttpClient(60 * 1000, PostDataFormat.RAW);
            Map<String, Object> header = buildHeader(HttpRequestMethod.POST, null, body);
            printCurl(HttpRequestMethod.POST, buildUrl(apiUrl), header, body);
            Result result = httpClient.post(header, body, buildUrl(apiUrl));

            JSONObject returnObj = checkResult(result, "查询虚拟机列表");

            List<JSONObject> results = JSONArray.parseArray(returnObj.getString("results"), JSONObject.class);
            if (ListTool.isEmpty(results)) {
                return null;
            }
            while (returnObj.getInteger("totalCount") > results.size()) {
                ++pageNo;
                body.put("pageNo", pageNo);
                result = doPost(apiUrl, body);
                returnObj = checkResult(result, "查询虚拟机列表");
                results.addAll(JSONArray.parseArray(returnObj.getString("results"), JSONObject.class));
            }
            return results.stream().map(o -> o.getString("iD")).collect(Collectors.toList());
        } catch (Exception e) {
            log.error("查询云主机列表失败 ： " + e);
        }
        return new ArrayList<>();
    }

}
