package com.bocloud.mcs.controller;

import com.bocloud.mcs.entity.PrometheusHttpConfigBean;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.core.RegistryService;
import com.megatron.framework.core.Service;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;

/**
 * Prometheus http服务发现
 *
 * @author fanzhenxi
 * @version 5.6.0-SNAPSHOT
 * @date 2022/2/23 18:23
 */
@RestController
@Slf4j
public class PrometheusController {

    @Autowired
    private Service service;
    @Autowired
    private RegistryService registry;

    @GetMapping("/targets")
    public List<PrometheusHttpConfigBean> targets() {
        try {
            List<PrometheusHttpConfigBean> prometheusHttpConfigBeans = new ArrayList<>();
            List<String> serviceInstances = registry.getServices(service);
            List<String> servers = ListTool.isEmpty(serviceInstances) ? new ArrayList<>() : serviceInstances;
            Set<String> targets = new HashSet<>();
            Optional.ofNullable(servers).orElse(new ArrayList<>()).forEach(server -> {
                server = server.replaceAll("http://", "").replaceAll("https://", "");
                targets.add(server);
            });
            PrometheusHttpConfigBean prometheusHttpConfigBean = new PrometheusHttpConfigBean(targets);
            prometheusHttpConfigBeans.add(prometheusHttpConfigBean);
            return prometheusHttpConfigBeans;
        } catch (Exception e) {
            log.error("获取zookeeper列表中节点失败！");
            return null;
        }
    }

}
