package com.bocloud.mcs.service.internal;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.entity.KvmHost;
import com.bocloud.ims.enums.CloudProvider;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author zhangyf
 */
@RequiredArgsConstructor
@Component
public class CmpCloudVendorService {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    /**
     * 查询已开启告警的平台
     *
     * @return 平台列表
     */
    public List<CloudVendor> list(CloudProvider cloudProvider) {
        Map<String, Object> headers = new HashMap<>(4);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());

        List<Param> paramList = new ArrayList<>();
        paramList.add(new Param(MapTools.simpleMap("is_alarm", 1), Sign.EQ));
        paramList.add(new Param(MapTools.simpleMap("type", cloudProvider.name()), Sign.EQ));
        RemoteService remoteService = serviceFactory.build(Service.create(String.valueOf(BoCloudService.CMP.getName())),
                "/v1/vendors", HttpMethod.GET, headers, new Pager(1, Integer.MAX_VALUE, paramList, null).toMap(false));
        GeneralResult<?> result = remoteService.invoke();
        Assert.isTrue(result.isSuccess(), "list cloud vendor failed, error:" + result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(CloudVendor.class);
    }

    /**
     * 查询云平台下所有宿主机
     *
     * @return
     */
    public List<KvmHost> listKvmHost(Long vendorId) {
        Map<String, Object> headers = new HashMap<>(4);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());

        List<Param> paramList = new ArrayList<>();
        paramList.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
        RemoteService remoteService = serviceFactory.build(Service.create(String.valueOf(BoCloudService.CMP.getName())), "/plugins/kvm/v1/hosts", HttpMethod.GET, headers,
                new Pager(1, Integer.MAX_VALUE, paramList, null).toMap(false));
        GeneralResult result = remoteService.invoke();
        Assert.isTrue(result.isSuccess(), "get kvm host list failed , error:" + result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(KvmHost.class);
    }

}
